//***************************************************************************************
//
// 	  Name: legicnfc.CRE.1v1.twn.c
// 	  Date: 2015/06/15
// 	  Version: 1.01
// 	  Mod: AA
//
//	  Note: This script is made directly from App_CELP106_CRE_Electronic_Legic_Prime.c
//
//***************************************************************************************

#include <sys.twn.h>

const byte MAXIDBYTES = 8;
const byte IDBYTECNT = 7;
const byte SEGMENT_NO = 0;
const byte IDOFFSET	= 4;

byte ID[IDBYTECNT];
byte LastID[IDBYTECNT];


const byte MAXTXRXCOUNT = 7+16;
byte TXRXData[MAXTXRXCOUNT];
byte RXCount;

byte IDLength;

byte TXRX(byte TXLength)
{
    if (!LegicGeneric(TXRXData,TXLength,TXRXData,RXCount,MAXTXRXCOUNT,20))
        return FALSE;
    // Command successful completed?
    return TXRXData[1] == 0;
}

byte SearchTag()
{
    // Go Idle
    TXRXData[0] = 0x00;
    TXRXData[1] = 0x02;
    if (!TXRX(2))
        return FALSE;
		
    // Is there a tag?
    TXRXData[0] = 0x01;
    TXRXData[1] = 0x00;
    if (!TXRX(2))
        return FALSE;
		
    // Expect one tag in the field.
    if (TXRXData[2] != 1)
        return FALSE;
		
	IDLength = TXRXData[3];	
    // Expect a tag with 4 bytes UID (Legic Prime)
    if (IDLength != 4)
        return FALSE;
	
    // Select this tag
    TXRXData[0] = 0x02;
    TXRXData[1] = 0x00;
    TXRXData[2] = 0x00;
    TXRXData[3] = 0x00;
    TXRXData[4] = 0x00; 
    if (!TXRX(5))
        return FALSE;

    // Search Segment
    TXRXData[0] = 0x03;
    TXRXData[1] = SEGMENT_NO;
    TXRXData[2] = 0x40;
    TXRXData[3] = 0x00; // Stamp length
    if (!TXRX(4))           
        return FALSE;

	// Read data
    TXRXData[0] = 0x05;
    TXRXData[1] = 0x00;
    TXRXData[2] = 0x00; 		// Address
    TXRXData[3] = IDBYTECNT; 	// Length
    if (!TXRX(4))
        return FALSE;
	   
	CopyBytes(ID,TXRXData[3],IDBYTECNT);
    return TRUE;
}

void main()
{
    // Make some noise at startup at minimum volume
    Beep(BEEPSUCCESS);
    // Set maximum volume
    SetVolume(4);
    // A V24 device is sending the version at startup
    if (GetConnection() == V24)
    {
        HostSendVersion();
        HostSendChar('\r');
    }
    // Turn on green LED
    LEDSet(GREEN,ON);
    // Turn off red LED
    LEDSet(RED,OFF);
    // No transponder found up to now
    FillBytes(LastID,0,MAXIDBYTES);
	
    while (TRUE)
    {
        // Search a transponder
        if (SearchTag())
        {
            // Is this transponder new to us?
            if (!CompBytes(ID,LastID,IDBYTECNT))
            {
                // Save this as new known ID
                CopyBytes(LastID,ID,IDBYTECNT);
				
                // Sound a beep
                Beep(BEEPHIGH);
                // Turn off the green LED
                LEDSet(GREEN,OFF);
                // Let the red one blink
                LEDSet(RED,BLINK);
				
				
				// Send ID number in hexadecimal for Username
				HostSendHex(ID[IDOFFSET],24,6);
				HostSendChar('\r');
				HostSendChar('\n');
				// Send ID number in hexadecimal for Password
				HostSendHex(ID[IDOFFSET],24,6);
				HostSendChar('\r');
				HostSendChar('\n');
            }
            // Start a timeout of a half second
            StartTimer(0,05);
        }
        if (TestTimer(0))
        {
            LEDSet(GREEN,ON);
            LEDSet(RED,OFF);
            FillBytes(LastID,0,MAXIDBYTES);
        }
    }
}
